@@ -249,11 +249,10 @@ export class Application extends Adw.Application {
         GeocodeFactory.getGeocoder().search(query, null, null, cancellable,
                                             (places, error) => {
             if (error) {
-                Utils.showDialog(_("An error has occurred"),
-                                 Gtk.MessageType.ERROR, this._mainWindow);
+                this._mainWindow.showToast(_("An error has occurred"));
             } else {
                 // clear search entry
-                this._mainWindow.placeEntry.text = '';
+                this._mainWindow.searchBar.placeEntry.text = '';
 
                 if (places) {
                     /* if there's only one place in results, show it directly
@@ -267,21 +266,19 @@ export class Application extends Adw.Application {
                         Application.normalStartup = false;
                         this._mainWindow.mapView.showPlace(places[0], true);
                     } else {
-                        this._mainWindow.placeEntry.grab_focus();
-                        this._mainWindow.placeEntry.updateResults(places, query,
-                                                                  false);
+                        this._mainWindow.searchBar.placeEntry.grab_focus();
+                        this._mainWindow.searchBar.updateResults(places, query,
+                                                                 false);
                     }
                 } else {
-                    Utils.showDialog(_("No results found"),
-                                     Gtk.MessageType.INFO, this._mainWindow);
+                    this._mainWindow.showToast(_("No results found"));
                 }
             }
         });
     }
 
     _invalidMapsUri(uri) {
-        Utils.showDialog(_("Invalid maps: URI: %s").format(uri),
-                         Gtk.MessageType.ERROR, this._mainWindow);
+        this._mainWindow.showToast(_("Invalid maps: URI: %s").format(uri));
     }
 
     vfunc_open(files) {
@@ -362,7 +359,7 @@ export class Application extends Adw.Application {
     vfunc_shutdown() {
         // need to unparent popover children to avoid GTK warnings on exit
         if (this._mainWindow) {
-            this._mainWindow.placeEntry.popover.unparent();
+            this._mainWindow.searchBar.placeEntry.popover.unparent();
             this._mainWindow.sidebar.unparentSearchPopovers();
         }
 
