@@ -36,27 +36,27 @@ import {FavoritesPopover} from './favoritesPopover.js';
 import * as Geoclue from './geoclue.js';
 import * as GeocodeFactory from './geocode.js';
 import {HeaderBarLeft, HeaderBarRight} from './headerBar.js';
-import {LocationServiceDialog} from './locationServiceDialog.js';
 import {MapView} from './mapView.js';
 import {PlaceBar} from './placeBar.js';
-import {PlaceEntry} from './placeEntry.js';
 import {PlaceStore} from './placeStore.js';
 import {PrintOperation} from './printOperation.js';
+import {SearchBar} from './searchBar.js';
 import * as Service from './service.js';
 import {ShapeLayer} from './shapeLayer.js';
 import {Sidebar} from './sidebar.js';
 import * as Utils from './utils.js';
+import {ZoomAndRotateControls} from './zoomAndRotateControls.js';
 
 const _ = gettext.gettext;
 
 const _CONFIGURE_ID_TIMEOUT = 100; // msecs
-const _ADAPTIVE_VIEW_WIDTH = 700;
-const _PLACE_ENTRY_MARGIN = 35;
+const _PLACE_ENTRY_MARGIN = 36;
 
 class ShapeLayerFileChooser extends Gtk.FileChooserNative {
 
     constructor(params) {
         super(params);
+
         let allFilter = new Gtk.FileFilter();
         allFilter.set_name(_("All Layer Files"));
         this.add_filter(allFilter);
@@ -80,14 +80,14 @@ GObject.registerClass({
     Template: 'resource:///org/gnome/Maps/ui/shape-layer-file-chooser.ui'
 }, ShapeLayerFileChooser);
 
-export class MainWindow extends Gtk.ApplicationWindow {
+export class MainWindow extends Adw.ApplicationWindow {
 
     get mapView() {
         return this._mapView;
     }
 
-    get placeEntry() {
-        return this._placeEntry;
+    get searchBar() {
+        return this._searchBar;
     }
 
     get sidebar() {
@@ -106,7 +106,14 @@ export class MainWindow extends Gtk.ApplicationWindow {
             hexpand: true,
             vexpand: true });
 
-        this._grid.attach(this._mapView, 0, 0, 1, 1);
+        this._mapOverlay.child = this._mapView;
+
+        this._mapOverlay.add_overlay(
+            new ZoomAndRotateControls({ mapView: this._mapView,
+                                        halign:  Gtk.Align.START,
+                                        valign:  Gtk.Align.START,
+                                        margin_start: 6,
+                                        margin_top: 6 }));
 
         this._mapView.gotoUserLocation(false);
 
@@ -122,7 +129,10 @@ export class MainWindow extends Gtk.ApplicationWindow {
         this._initDND();
         this._initPlaceBar();
 
-        this._grid.attach(this._sidebar, 1, 0, 1, 2);
+        this._splitView.sidebar = this._sidebar;
+        this._splitView.connect('notify::show-sidebar', () => {
+            this._setRevealSidebar(this._splitView.show_sidebar);
+        });
 
         /* for some reason, setting the title of the window through the .ui
          * template does not work anymore (maybe has something to do with
@@ -131,32 +141,40 @@ export class MainWindow extends Gtk.ApplicationWindow {
          * overview.
          */
         this.title = _("Maps");
+
+        this._breakpoint.connect('apply', () => this._onBreakpointApplied());
+        this._breakpoint.connect('unapply', () => this._onBreakpointUnapplied());
     }
 
-    _createPlaceEntry() {
-        let placeEntry = new PlaceEntry({ mapView: this._mapView,
-                                          visible: true,
-                                          margin_start: _PLACE_ENTRY_MARGIN,
-                                          margin_end: _PLACE_ENTRY_MARGIN,
-                                          max_width_chars: 50,
-                                          matchRoute: true });
-        placeEntry.connect('notify::place', () => {
-            if (placeEntry.place) {
-                this._mapView.showPlace(placeEntry.place, true);
+    showToast(message) {
+        Utils.showToastInOverlay(message, this._overlay);
+    }
+
+    addToast(toast) {
+        this._overlay.add_toast(toast);
+    }
+
+    _createSearchBar() {
+        let searchBar = new SearchBar({ mapView: this._mapView,
+                                        margin_start: _PLACE_ENTRY_MARGIN,
+                                        margin_end: _PLACE_ENTRY_MARGIN });
+        searchBar.connect('notify::place', () => {
+            if (searchBar.place) {
+                this._mapView.showPlace(searchBar.place, true);
             }
         });
 
-        let popover = placeEntry.popover;
+        let popover = searchBar.popover;
         popover.connect('selected', () => this._mapView.grab_focus());
 
         this._buttonPressGesture = new Gtk.GestureSingle();
         this._mapView.add_controller(this._buttonPressGesture);
         this._buttonPressGesture.connect('begin', () => popover.popdown());
-        return placeEntry;
+        return searchBar;
     }
 
     _createSidebar() {
-        let sidebar = new Sidebar(this._mapView);
+        let sidebar = new Sidebar({ mapView: this._mapView });
 
         Application.routeQuery.connect('notify', () => this._setRevealSidebar(true));
 
@@ -208,13 +226,25 @@ export class MainWindow extends Gtk.ApplicationWindow {
                 onChangeState: (a, v) => this._onToggleSidebarChangeState(a, v)
             },
             'zoom-in': {
-                accels: ['plus', '<Primary>plus', 'KP_Add', '<Primary>KP_Add', 'equal', '<Primary>equal'],
+                accels: ['<Primary>plus', 'KP_Add', '<Primary>KP_Add', '<Primary>equal'],
                 onActivate: () => this._mapView.zoomIn()
             },
             'zoom-out': {
-                accels: ['minus', '<Primary>minus', 'KP_Subtract', '<Primary>KP_Subtract'],
+                accels: ['<Primary>minus', 'KP_Subtract', '<Primary>KP_Subtract'],
                 onActivate:  () => this._mapView.zoomOut()
             },
+            'rotate-clockwise': {
+                accels: ['<Primary>Right'],
+                onActivate: () => this._rotateMap(Math.PI / 32)
+            },
+            'rotate-counter-clockwise': {
+                accels: ['<Primary>Left'],
+                onActivate: () => this._rotateMap(-Math.PI / 32)
+            },
+            'reset-rotation': {
+                accels: ['<Primary>Up'],
+                onActivate: () => { this._mapView.map.viewport.rotation = 0.0; }
+            },
             'show-scale': {
                 accels: ['<Primary>S'],
                 paramType: 'b',
@@ -222,7 +252,15 @@ export class MainWindow extends Gtk.ApplicationWindow {
             },
             'find': {
                 accels: ['<Primary>F'],
-                onActivate: () => this._placeEntry.grab_focus()
+                onActivate: () => this._onFindActivate()
+            },
+            'browse': {
+                accels: ['<Shift><Primary>F'],
+                onActivate: () => this._onBrowseActivate()
+            },
+            'show-search-results': {
+                accels: ['<Primary>R'],
+                onActivate: () => this._onShowSearchResultsActivate()
             },
             'print-route': {
                 accels: ['<Primary>P'],
@@ -254,6 +292,17 @@ export class MainWindow extends Gtk.ApplicationWindow {
             };
         }
 
+        if (Shumate.VectorRenderer.is_supported()) {
+            actions['enable-vector-layer'] = {
+                state: ['b', Application.settings.get('map-type') === MapView.MapType.VECTOR],
+                onChangeState: (action, variant) => {
+                    action.set_state(variant);
+                    const state = variant.get_boolean();
+                    this._mapView.setMapType(state ? MapView.MapType.VECTOR : MapView.MapType.STREET);
+                }
+            };
+        }
+
         Utils.addActions(this, actions, Application.settings);
     }
 
@@ -261,18 +310,7 @@ export class MainWindow extends Gtk.ApplicationWindow {
         this.connect('close-request', () => this._quit());
         this.connect('notify::default-width', () => this._onSizeChanged());
         this.connect('notify::default-height', () => this._onSizeChanged());
-
         this.connect('notify::maximized', () => this._onMaximizedChanged());
-        // TODO: GTK4, is this needed?
-        /*
-        this._mapView.view.connect('button-press-event', () => {
-            // Can not call something that will generate clutter events
-            // from a clutter event-handler. So use an idle.
-            GLib.idle_add(null, () => this._mapView.grab_focus());
-        });
-        */
-
-        //this._placeEntry.set_key_capture_widget(this)
 
         let viewport = this._mapView.map.viewport;
 
@@ -317,8 +355,9 @@ export class MainWindow extends Gtk.ApplicationWindow {
         this._headerBarRight = new HeaderBarRight({ mapView: this._mapView });
         this._headerBar.pack_end(this._headerBarRight);
 
-        this._placeEntry = this._createPlaceEntry();
-        this._headerBar.title_widget = this._placeEntry;
+        this._searchBar = this._createSearchBar();
+        this._headerBar.title_widget = this._searchBar;
+        this._searchBar.placeEntry.grab_focus();
 
         Application.geoclue.connect('notify::state',
                                     this._updateLocationSensitivity.bind(this));
@@ -331,44 +370,20 @@ export class MainWindow extends Gtk.ApplicationWindow {
         this._actionBar.pack_end(this._actionBarRight);
     }
 
-    vfunc_realize() {
-        super.vfunc_realize();
-        this._surface = this.get_native().get_surface();
-
-        // update adaptive status based on initial geometry
-        this._updateAdaptiveMode();
-
-        this._surface.connect('notify::width', () => {
-            this._updateAdaptiveMode();
-        });
+    _onBreakpointApplied() {
+        this.application.adaptive_mode = true;
+        this._headerBarLeft.hide();
+        this._headerBarRight.hide();
+        this._searchBar.set_margin_start(0);
+        this._searchBar.set_margin_end(0);
     }
 
-    _updateAdaptiveMode() {
-        let width = this._surface.width;
-
-        /* don't update adaptive state when the window has just been
-         * realized before adding widgets, having a tiny size
-         * this avoid flickering the adaptive mode bottom toolbar
-         * at startup in "desktop" mode when it shouldn't be
-         */
-        if (width < 10)
-            return;
-
-        if (width < _ADAPTIVE_VIEW_WIDTH) {
-            this.application.adaptive_mode = true;
-            this._headerBarLeft.hide();
-            this._headerBarRight.hide();
-            this._actionBarRevealer.set_reveal_child(true);
-            this._placeEntry.set_margin_start(0);
-            this._placeEntry.set_margin_end(0);
-        } else {
-            this.application.adaptive_mode = false;
-            this._headerBarLeft.show();
-            this._headerBarRight.show();
-            this._actionBarRevealer.set_reveal_child(false);
-            this._placeEntry.set_margin_start(_PLACE_ENTRY_MARGIN);
-            this._placeEntry.set_margin_end(_PLACE_ENTRY_MARGIN);
-        }
+    _onBreakpointUnapplied() {
+        this.application.adaptive_mode = false;
+        this._headerBarLeft.show();
+        this._headerBarRight.show();
+        this._searchBar.set_margin_start(_PLACE_ENTRY_MARGIN);
+        this._searchBar.set_margin_end(_PLACE_ENTRY_MARGIN);
     }
 
     _saveWindowGeometry() {
@@ -421,6 +436,24 @@ export class MainWindow extends Gtk.ApplicationWindow {
         return false;
     }
 
+    _onFindActivate() {
+        let placeEntry = this._searchBar.placeEntry;
+
+        placeEntry.grab_focus();
+        placeEntry.select_region(0, placeEntry.text.length);
+    }
+
+    _onBrowseActivate() {
+        this._searchBar.placeEntry.browsePois();
+    }
+
+    _onShowSearchResultsActivate() {
+        if (this._searchBar.placeEntry.popover.numResults > 0) {
+            this._searchBar.placeEntry.popover.showResult();
+            this._searchBar.placeEntry.grab_focus();
+        }
+    }
+
     _onGotoUserLocationActivate() {
         let message;
 
@@ -432,18 +465,29 @@ export class MainWindow extends Gtk.ApplicationWindow {
         Application.geoclue.start(() => {
             switch(Application.geoclue.state) {
             case Geoclue.State.FAILED:
-                message = _("Failed to connect to location service");
-                Utils.showDialog(message, Gtk.MessageType.ERROR, this);
+                this.showToast(_("Failed to connect to location service"));
                 break;
 
             case Geoclue.State.DENIED:
-                let dialog = new LocationServiceDialog({
-                    visible: true,
-                    transient_for: this,
-                    modal: true });
+                let locationServiceToast =
+                    new Adw.Toast({ title: _("Turn on location services"),
+                                    button_label: _("Location Settings") });
+
+                locationServiceToast.connect('button-clicked', () => {
+                    let privacyInfo =
+                        Gio.DesktopAppInfo.new('gnome-location-panel.desktop');
+
+                    try {
+                        let display = Gdk.Display.get_default();
+
+                        privacyInfo.launch([], display.get_app_launch_context());
+                    } catch(e) {
+                        Utils.debug('launching privacy panel failed: ' + e);
+                    }
+                });
+
+                this.addToast(locationServiceToast);
 
-                dialog.connect('response', () => dialog.destroy());
-                dialog.present();
                 break;
 
             default:
@@ -476,6 +520,26 @@ export class MainWindow extends Gtk.ApplicationWindow {
         dialog.show();
     }
 
+    _rotateMap(angle) {
+        let rotation = this._mapView.map.viewport.rotation;
+
+        rotation += angle;
+
+        // keep the rotation in [0..2 * PI)
+        if (rotation < 0)
+            rotation += 2 * Math.PI
+        else if (rotation >= 2 * Math.PI)
+            rotation -= 2 * Math.PI;
+
+        /* if the resulting angle is close to 0, snap back to 0 to avoid
+         * rounding errors adding when doing multiple rotations
+         */
+        if (rotation < 0.01 || 2 * Math.PI - rotation < 0.01)
+            rotation = 0;
+
+        this._mapView.map.viewport.rotation = rotation;
+    }
+
     _printRouteActivate() {
         if (this._mapView.routeShowing) {
             let operation = new PrintOperation({ mainWindow: this });
@@ -502,7 +566,10 @@ export class MainWindow extends Gtk.ApplicationWindow {
         action.set_state(variant);
 
         let reveal = variant.get_boolean();
-        this._sidebar.set_reveal_child(reveal);
+        this._splitView.show_sidebar = reveal;
+
+        if (reveal)
+            this._splitView.sidebar.focusStartEntry();
     }
 
     _setRevealSidebar(value) {
@@ -523,10 +590,10 @@ export class MainWindow extends Gtk.ApplicationWindow {
             /* Translators: This is the program name. */
             application_name: _("Maps"),
             application_icon: pkg.name,
-            copyright: _("Copyright © 2011 – 2022 Red Hat, Inc. and The GNOME Maps authors"),
+            copyright: _("Copyright © 2011 – 2023 Red Hat, Inc. and The GNOME Maps authors"),
             license_type: Gtk.License.GPL_2_0,
             version: pkg.version,
-            website: 'https://live.gnome.org/Apps/Maps',
+            website: 'https://apps.gnome.org/app/org.gnome.Maps',
             issue_url: 'https://gitlab.gnome.org/GNOME/gnome-maps/-/issues/new',
             transient_for: this
         });
@@ -626,6 +693,9 @@ GObject.registerClass({
                         'mainMenuButton',
                         'grid',
                         'actionBar',
-                        'actionBarRevealer',
-                        'placeBarContainer']
+                        'placeBarContainer',
+                        'overlay',
+                        'mapOverlay',
+                        'splitView',
+                        'breakpoint' ]
 }, MainWindow);
