@@ -30,12 +30,15 @@ const TYPE_ICON_MAP = {
         aerodrome: 'route-transit-airplane-symbolic'
     },
     amenity: {
+        atm:              'coin-symbolic',
         bank:             'bank-symbolic',
         bar:              'bar-symbolic',
         bbq:              'barbecue-symbolic',
+        bicycle_parking:  'route-bike-symbolic',
         biergarten:       'pub-symbolic',
         bus_station:      'route-transit-bus-symbolic',
         cafe:             'cafe-symbolic',
+        car_rental:       'route-car-symbolic',
         charging_station: 'electric-car-symbolic',
         cinema:           'video-camera-symbolic',
         clinic:           'hospital-sign-symbolic',
@@ -50,13 +53,17 @@ const TYPE_ICON_MAP = {
         hospital:         'hospital-symbolic',
         kindergarten:     'school-symbolic',
         library:          'library-symbolic',
-        night_club:       'bar-symbolic',
+        luggage_locker:   'briefcase-symbolic',
+        nightclub:        'music-note-symbolic',
         parking:          'parking-sign-symbolic',
+        police:           'police-badge2-symbolic',
+        post_box:         'post-box-symbolic',
         post_office:      'post-box-symbolic',
         pub:              'pub-symbolic',
         restaurant:       'restaurant-symbolic',
         school:           'school-symbolic',
         theatre:          'theater-symbolic',
+        toilets:          'toilets-symbolic',
         university:       'school-symbolic',
         veterinary:       'dog-symbolic'
     },
@@ -76,7 +83,10 @@ const TYPE_ICON_MAP = {
     },
     leisure: {
         dog_park:         'dog-symbolic',
+        fitness_centre:   'weight2-symbolic',
+        fitness_station:  'weight2-symbolic',
         garden:           'tree-symbolic',
+        golf_course:      'golf-symbolic',
         nature_reserve:   'tree-symbolic',
         park:             'tree-symbolic',
         pitch:            'baseball-symbolic',
@@ -119,9 +129,11 @@ const TYPE_ICON_MAP = {
         bakery:           'bread-symbolic',
         convenience:      'shopping-cart-symbolic',
         department_store: 'shopping-cart-symbolic',
+        electronics:      'smartphone-symbolic',
         general:          'shopping-cart-symbolic',
         mall:             'shopping-cart-symbolic',
         supermarket:      'shopping-cart-symbolic',
+        ticket:           'ticket-symbolic',
         wine:             'drinks-symbolic'
     },
     tourism: {
